@@ -2,14 +2,17 @@ package driver
 
 import (
 	"context"
+	"fmt"
 	"net"
 	"os"
+	"sort"
 	"strings"
 	"time"
+	"strconv"
 
-	log "github.com/Sirupsen/logrus"
 	"github.com/pkg/errors"
 	libcalicoErrors "github.com/projectcalico/libcalico-go/lib/errors"
+	log "github.com/sirupsen/logrus"
 
 	dockerClient "github.com/docker/docker/client"
 	"github.com/docker/go-plugins-helpers/network"
@@ -27,6 +30,7 @@ const DOCKER_LABEL_PREFIX = "org.projectcalico.label."
 const LABEL_POLL_TIMEOUT_ENVKEY = "CALICO_LIBNETWORK_LABEL_POLL_TIMEOUT"
 const CREATE_PROFILES_ENVKEY = "CALICO_LIBNETWORK_CREATE_PROFILES"
 const LABEL_ENDPOINTS_ENVKEY = "CALICO_LIBNETWORK_LABEL_ENDPOINTS"
+const VETH_MTU_ENVKEY = "CALICO_LIBNETWORK_VETH_MTU"
 
 // NetworkDriver is the Calico network driver representation.
 // Must be used with Calico IPAM and supports IPv4 only.
@@ -40,6 +44,8 @@ type NetworkDriver struct {
 
 	DummyIPV4Nexthop string
 
+	vethMTU uint16
+
 	labelPollTimeout time.Duration
 
 	createProfiles bool
@@ -70,6 +76,20 @@ func NewNetworkDriver(client *datastoreClient.Client) network.Driver {
 		labelEndpoints: strings.EqualFold(os.Getenv(LABEL_ENDPOINTS_ENVKEY), "true"),
 	}
 
+	// Check if MTU environment variable is given, parse into uint16
+	// and override the default in the NetworkDriver.
+	if mtuStr, ok := os.LookupEnv(VETH_MTU_ENVKEY); ok {
+		mtu, err := strconv.ParseUint(mtuStr, 10, 16)
+		if err != nil {
+			log.Fatalf("Failed to parse %v '%v' into uint16: %v",
+				VETH_MTU_ENVKEY, mtuStr, err)
+		}
+
+		driver.vethMTU = uint16(mtu)
+
+		log.WithField("mtu", mtu).Info("Parsed veth MTU")
+	}
+
 	if !driver.createProfiles {
 		log.Info("Feature disabled: no Calico profiles will be created per network")
 	}
@@ -86,33 +106,101 @@ func (d NetworkDriver) GetCapabilities() (*network.CapabilitiesResponse, error)
 	return &resp, nil
 }
 
+// AllocateNetwork is used for swarm-mode support in remote plugins, which
+// Calico's libnetwork-plugin doesn't currently support.
+func (d NetworkDriver) AllocateNetwork(request *network.AllocateNetworkRequest) (*network.AllocateNetworkResponse, error) {
+	var resp network.AllocateNetworkResponse
+	logutils.JSONMessage("AllocateNetwork response", resp)
+	return &resp, nil
+}
+
+// FreeNetwork is used for swarm-mode support in remote plugins, which
+// Calico's libnetwork-plugin doesn't currently support.
+func (d NetworkDriver) FreeNetwork(request *network.FreeNetworkRequest) error {
+	logutils.JSONMessage("FreeNetwork request", request)
+	return nil
+}
+
 func (d NetworkDriver) CreateNetwork(request *network.CreateNetworkRequest) error {
 	logutils.JSONMessage("CreateNetwork", request)
+	knownOpts := map[string]bool{"com.docker.network.enable_ipv6": true}
+	// Reject all options (--internal, --enable_ipv6, etc)
+	for k, v := range request.Options {
+		skip := false
+		for known, _ := range knownOpts {
+			if k == known {
+				skip = true
+				break
+			}
+		}
+		if skip {
+			continue
+		}
+		optionSet := false
+		flagName := k
+		flagValue := fmt.Sprintf("%s", v)
+		multipleFlags := false
+		switch v := v.(type) {
+		case bool:
+			// if v == true then optionSet = true
+			optionSet = v
+			flagName = "--" + strings.TrimPrefix(k, "com.docker.network.")
+			flagValue = ""
+			break
+		case map[string]interface{}:
+			optionSet = len(v) != 0
+			flagName = ""
+			numFlags := 0
+			// Sort flags for consistent error reporting
+			flags := []string{}
+			for flag := range v {
+				flags = append(flags, flag)
+			}
+			sort.Strings(flags)
 
-	genericOpts, ok := request.Options["com.docker.network.generic"]
-	if ok {
-		opts, ok := genericOpts.(map[string]interface{})
-		if ok && len(opts) != 0 {
-			err := errors.New("Arbitrary options are not supported")
-			log.Println(err)
+			for _, flag := range flags {
+				flagName = flagName + flag + ", "
+				numFlags++
+			}
+			multipleFlags = numFlags > 1
+			flagName = strings.TrimSuffix(flagName, ", ")
+			flagValue = ""
+			break
+		default:
+			// for unknown case let optionSet = true
+			optionSet = true
+		}
+		if optionSet {
+			if flagValue != "" {
+				flagValue = " (" + flagValue + ")"
+			}
+			f := "flag"
+			if multipleFlags {
+				f = "flags"
+			}
+			err := errors.New("Calico driver does not support the " + f + " " + flagName + flagValue + ".")
+			log.Errorln(err)
 			return err
 		}
 	}
 
-	// Calico driver does not allow you use the --internal flag
-	internal, ok := request.Options["com.docker.network.internal"].(bool)
-	if ok && internal {
-		err := errors.New("Calico driver does not support the --internal flag.")
-		log.Errorln(err)
-		return err
-	}
-
 	for _, ipData := range request.IPv4Data {
 		// Older version of Docker have a bug where they don't provide the correct AddressSpace
 		// so we can't check for calico IPAM using our known address space.
+		// The Docker issue, https://github.com/projectcalico/libnetwork-plugin/issues/77,
+		// was fixed sometime between 1.11.2 and 1.12.3.
 		// Also the pool might not have a fixed values if --subnet was passed
 		// So the only safe thing is to check for our special gateway value
 		if ipData.Gateway != "0.0.0.0/0" {
+			err := errors.New("Non-Calico IPAM driver is used. Note: Docker before 1.12.3 is unsupported")
+			log.Errorln(err)
+			return err
+		}
+	}
+
+	for _, ipData := range request.IPv6Data {
+		// Don't support older versions of Docker which have a bug where the correct AddressSpace isn't provided
+		if ipData.AddressSpace != CalicoGlobalAddressSpace {
 			err := errors.New("Non-Calico IPAM driver is used")
 			log.Errorln(err)
 			return err
@@ -139,7 +227,7 @@ func (d NetworkDriver) CreateEndpoint(request *network.CreateEndpointRequest) (*
 	}
 
 	log.Debugf("Creating endpoint %v\n", request.EndpointID)
-	if request.Interface.Address == "" {
+	if request.Interface.Address == "" && request.Interface.AddressIPv6 == "" {
 		err := errors.New("No address assigned for endpoint")
 		log.Errorln(err)
 		return nil, err
@@ -160,6 +248,18 @@ func (d NetworkDriver) CreateEndpoint(request *network.CreateEndpointRequest) (*
 		addresses = append(addresses, caliconet.IPNet{IPNet: net.IPNet{IP: ip4, Mask: net.CIDRMask(32, 32)}})
 	}
 
+	if request.Interface.AddressIPv6 != "" {
+		// Parse the address this function was passed.
+		ip6, ipnet, err := net.ParseCIDR(request.Interface.AddressIPv6)
+		log.Debugf("Parsed IP %v from (%v) \n", ip6, request.Interface.AddressIPv6)
+		if err != nil {
+			err = errors.Wrapf(err, "Parsing %v as CIDR failed", request.Interface.AddressIPv6)
+			log.Errorln(err)
+			return nil, err
+		}
+		addresses = append(addresses, caliconet.IPNet{IPNet: *ipnet})
+	}
+
 	endpoint := api.NewWorkloadEndpoint()
 	endpoint.Metadata.Node = hostname
 	endpoint.Metadata.Orchestrator = d.orchestratorID
@@ -230,9 +330,9 @@ func (d NetworkDriver) CreateEndpoint(request *network.CreateEndpointRequest) (*
 
 	log.Debugf("Workload created, data: %+v\n", endpoint)
 
-    if d.labelEndpoints {
-        go d.populateWorkloadEndpointWithLabels(request.NetworkID, endpoint)
-    }
+	if d.labelEndpoints {
+		go d.populateWorkloadEndpointWithLabels(request.NetworkID, endpoint)
+	}
 
 	var endpointInterface network.EndpointInterface
 	if !userProvidedMac {
@@ -295,10 +395,10 @@ func (d NetworkDriver) Join(request *network.JoinRequest) (*network.JoinResponse
 	hostInterfaceName := "cali" + prefix
 	tempInterfaceName := "temp" + prefix
 
-	if err = netns.CreateVeth(hostInterfaceName, tempInterfaceName); err != nil {
+	if err = netns.CreateVeth(hostInterfaceName, tempInterfaceName, d.vethMTU); err != nil {
 		err = errors.Wrapf(
-			err, "Veth creation error, hostInterfaceName=%v, tempInterfaceName=%v",
-			hostInterfaceName, tempInterfaceName)
+			err, "Veth creation error, hostInterfaceName=%v, tempInterfaceName=%v, vethMTU=%v",
+			hostInterfaceName, tempInterfaceName, d.vethMTU)
 		log.Errorln(err)
 		return nil, err
 	}
@@ -322,6 +422,19 @@ func (d NetworkDriver) Join(request *network.JoinRequest) (*network.JoinResponse
 		NextHop:     "",
 	})
 
+	linkLocalAddr := netns.GetLinkLocalAddr(hostInterfaceName)
+	if linkLocalAddr == nil {
+		log.Warnf("No IPv6 link local address for %s", hostInterfaceName)
+	} else {
+		resp.GatewayIPv6 = fmt.Sprintf("%s", linkLocalAddr)
+		nextHopIPv6 := fmt.Sprintf("%s/128", linkLocalAddr)
+		resp.StaticRoutes = append(resp.StaticRoutes, &network.StaticRoute{
+			Destination: nextHopIPv6,
+			RouteType:   1, // 1 = CONNECTED
+			NextHop:     "",
+		})
+	}
+
 	logutils.JSONMessage("Join response", resp)
 
 	return resp, nil
@@ -341,7 +454,7 @@ func (d NetworkDriver) DiscoverNew(request *network.DiscoveryNotification) error
 }
 
 func (d NetworkDriver) DiscoverDelete(request *network.DiscoveryNotification) error {
-	logutils.JSONMessage("DiscoverNew", request)
+	logutils.JSONMessage("DiscoverDelete", request)
 	log.Debugln("DiscoverDelete response JSON={}")
 	return nil
 }
