@@ -4,6 +4,7 @@ import (
 	"fmt"
 	"math/rand"
 	"regexp"
+	"os"
 
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
@@ -16,7 +17,8 @@ import (
 var _ = Describe("Libnetwork Tests", func() {
 	BeforeEach(func() {
 		WipeEtcd()
-		CreatePool("192.169.0.0/16")
+		CreatePool("192.169.0.0/16", false)
+		CreatePool("2001:db8::/32", true)
 	})
 
 	// Run the plugin just once for all tests in this file.
@@ -48,7 +50,7 @@ var _ = Describe("Libnetwork Tests", func() {
 			It("rejects --internal being used", func() {
 				session := DockerSession("docker network create $RANDOM --internal -d calico --ipam-driver calico-ipam")
 				Eventually(session).Should(Exit(1))
-				Eventually(session.Err).Should(Say("Error response from daemon: NetworkDriver.CreateNetwork: Calico driver does not support the --internal flag."))
+				Eventually(session.Err).Should(Say("Error response from daemon: NetworkDriver.CreateNetwork: Calico driver does not support the flag --internal."))
 			})
 			It("rejects --ip-range being used", func() {
 				session := DockerSession("docker network create $RANDOM --ip-range 192.169.1.0/24 --subnet=192.169.0.0/16 -d calico --ipam-driver calico-ipam")
@@ -63,7 +65,12 @@ var _ = Describe("Libnetwork Tests", func() {
 			It("rejects --opt being used", func() {
 				session := DockerSession("docker network create $RANDOM --opt REJECT -d calico --ipam-driver calico-ipam")
 				Eventually(session).Should(Exit(1))
-				Eventually(session.Err).Should(Say("Error response from daemon: NetworkDriver.CreateNetwork: Arbitrary options are not supported"))
+				Eventually(session.Err).Should(Say("NetworkDriver.CreateNetwork: Calico driver does not support the flag REJECT."))
+			})
+			It("rejects multiple --opt being used", func() {
+				session := DockerSession("docker network create $RANDOM --opt REJECT --opt REJECT2 -d calico --ipam-driver calico-ipam")
+				Eventually(session).Should(Exit(1))
+				Eventually(session.Err).Should(Say("NetworkDriver.CreateNetwork: Calico driver does not support the flags REJECT, REJECT2."))
 			})
 		})
 		Context("checking success cases", func() {
@@ -77,8 +84,9 @@ var _ = Describe("Libnetwork Tests", func() {
 				session := DockerSession("docker network create success$RANDOM --subnet 192.169.0.0/16 -d calico --ipam-driver calico-ipam")
 				Eventually(session).Should(Exit(0))
 			})
-
-			PIt("creates a network with IPv6", func() { //TODO - IPv6 support
+			It("creates a network with IPv6", func() {
+				session := DockerSession("docker network create --ipv6 success$RANDOM -d calico --ipam-driver calico-ipam")
+				Eventually(session).Should(Exit(0))
 			})
 			PIt("creates a network with IPv6 from a specific subnet", func() {
 			})
@@ -130,7 +138,7 @@ var _ = Describe("Libnetwork Tests", func() {
 
 		It("creates a container on a network  and checks all assertions", func() {
 			// Create a container that will just sit in the background
-			DockerString(fmt.Sprintf("docker run --net %s -tid --name %s busybox", name, name))
+			DockerString(fmt.Sprintf("docker run --net %s -tid --name %s %s", name, name, os.Getenv("BUSYBOX_IMAGE") ))
 
 			// Gather information for assertions
 			docker_endpoint := GetDockerEndpoint(name, name)
@@ -145,7 +153,7 @@ var _ = Describe("Libnetwork Tests", func() {
 			// Check that the endpoint is created in etcd
 			etcd_endpoint := GetEtcdString(fmt.Sprintf("/calico/v1/host/test/workload/libnetwork/libnetwork/endpoint/%s", endpoint_id))
 			Expect(etcd_endpoint).Should(MatchJSON(fmt.Sprintf(
-				`{"state":"active","name":"%s","mac":"%s","profile_ids":["%s"],"ipv4_nets":["%s/32"],"ipv6_nets":[]}`,
+				`{"state":"active","name":"%s","active_instance_id":"","mac":"%s","profile_ids":["%s"],"ipv4_nets":["%s/32"],"ipv6_nets":[]}`,
 				interface_name, mac, name, ip)))
 
 			// Check profile
@@ -167,7 +175,7 @@ var _ = Describe("Libnetwork Tests", func() {
 
 			// Make sure the container has the routes we expect
 			routes := DockerString(fmt.Sprintf("docker exec -i %s ip route", name))
-			Expect(routes).Should(Equal("default via 169.254.1.1 dev cali0 \n169.254.1.1 dev cali0"))
+			Expect(routes).Should(Equal("default via 169.254.1.1 dev cali0 \n169.254.1.1 dev cali0 scope link"))
 
 			// Delete container
 			DockerString(fmt.Sprintf("docker rm -f %s", name))
@@ -175,7 +183,7 @@ var _ = Describe("Libnetwork Tests", func() {
 		It("creates a container with specific MAC", func() {
 			// Create a container that will just sit in the background
 			chosen_mac := "00:22:33:44:55:66"
-			DockerString(fmt.Sprintf("docker run --mac-address %s --net %s -tid --name %s busybox", chosen_mac, name, name))
+			DockerString(fmt.Sprintf("docker run --mac-address %s --net %s -tid --name %s %s", chosen_mac, name, name, os.Getenv("BUSYBOX_IMAGE") ))
 
 			// Gather information for assertions
 			docker_endpoint := GetDockerEndpoint(name, name)
@@ -190,7 +198,7 @@ var _ = Describe("Libnetwork Tests", func() {
 			// Check that the endpoint is created in etcd
 			etcd_endpoint := GetEtcdString(fmt.Sprintf("/calico/v1/host/test/workload/libnetwork/libnetwork/endpoint/%s", endpoint_id))
 			Expect(etcd_endpoint).Should(MatchJSON(fmt.Sprintf(
-				`{"state":"active","name":"%s","mac":"%s","profile_ids":["%s"],"ipv4_nets":["%s/32"],"ipv6_nets":[]}`,
+				`{"state":"active","name":"%s","active_instance_id":"","mac":"%s","profile_ids":["%s"],"ipv4_nets":["%s/32"],"ipv6_nets":[]}`,
 				interface_name, mac, name, ip)))
 
 			// Check the interface exists on the Host - it has an autoassigned
@@ -204,7 +212,7 @@ var _ = Describe("Libnetwork Tests", func() {
 
 			// Make sure the container has the routes we expect
 			routes := DockerString(fmt.Sprintf("docker exec -i %s ip route", name))
-			Expect(routes).Should(Equal("default via 169.254.1.1 dev cali0 \n169.254.1.1 dev cali0"))
+			Expect(routes).Should(Equal("default via 169.254.1.1 dev cali0 \n169.254.1.1 dev cali0 scope link"))
 
 			// Delete container
 			DockerString(fmt.Sprintf("docker rm -f %s", name))
@@ -212,7 +220,7 @@ var _ = Describe("Libnetwork Tests", func() {
 
 		PIt("creates a container with specific link local address", func() { // https://github.com/docker/docker/issues/28606
 			// Create a container that will just sit in the background
-			DockerString(fmt.Sprintf("docker run --link-local-ip 169.254.0.50 %s --net %s -tid --name %s busybox", name, name, name))
+			DockerString(fmt.Sprintf("docker run --link-local-ip 169.254.0.50 %s --net %s -tid --name %s %s", name, name, name, os.Getenv("BUSYBOX_IMAGE") ))
 
 			// Delete container
 			DockerString(fmt.Sprintf("docker rm -f %s", name))
@@ -227,7 +235,7 @@ var _ = Describe("Libnetwork Tests", func() {
 			DockerString(fmt.Sprintf("docker network create %s --subnet 192.169.0.0/16 -d calico --ipam-driver calico-ipam", name_subnet))
 			// Create a container that will just sit in the background
 			chosen_ip := "192.169.50.51"
-			DockerString(fmt.Sprintf("docker run --ip %s --net %s -tid --name %s busybox", chosen_ip, name_subnet, name_subnet))
+			DockerString(fmt.Sprintf("docker run --ip %s --net %s -tid --name %s %s", chosen_ip, name_subnet, name_subnet, os.Getenv("BUSYBOX_IMAGE") ))
 
 			// Gather information for assertions
 			docker_endpoint := GetDockerEndpoint(name_subnet, name_subnet)
@@ -241,7 +249,7 @@ var _ = Describe("Libnetwork Tests", func() {
 			// Check that the endpoint is created in etcd
 			etcd_endpoint := GetEtcdString(fmt.Sprintf("/calico/v1/host/test/workload/libnetwork/libnetwork/endpoint/%s", endpoint_id))
 			Expect(etcd_endpoint).Should(MatchJSON(fmt.Sprintf(
-				`{"state":"active","name":"%s","mac":"%s","profile_ids":["%s"],"ipv4_nets":["%s/32"],"ipv6_nets":[]}`,
+				`{"state":"active","name":"%s","active_instance_id":"","mac":"%s","profile_ids":["%s"],"ipv4_nets":["%s/32"],"ipv6_nets":[]}`,
 				interface_name_subnet, mac, name_subnet, ip)))
 
 			// Check the interface exists on the Host - it has an autoassigned
@@ -255,7 +263,45 @@ var _ = Describe("Libnetwork Tests", func() {
 
 			// Make sure the container has the routes we expect
 			routes := DockerString(fmt.Sprintf("docker exec -i %s ip route", name_subnet))
-			Expect(routes).Should(Equal("default via 169.254.1.1 dev cali0 \n169.254.1.1 dev cali0"))
+			Expect(routes).Should(Equal("default via 169.254.1.1 dev cali0 \n169.254.1.1 dev cali0 scope link"))
+
+			// Delete container and network
+			DockerString(fmt.Sprintf("docker rm -f %s", name_subnet))
+			DockerString(fmt.Sprintf("docker network rm %s", name_subnet))
+		})
+
+		It("creates a container in network with a subnet", func() {
+			name_subnet := fmt.Sprintf("run%d", rand.Uint32())
+			DockerString(fmt.Sprintf("docker network create %s --subnet 192.169.0.0/16 -d calico --ipam-driver calico-ipam", name_subnet))
+			DockerString(fmt.Sprintf("docker run --net %s -tid --name %s %s", name_subnet, name_subnet, os.Getenv("BUSYBOX_IMAGE") ))
+
+			// Gather information for assertions
+			docker_endpoint := GetDockerEndpoint(name_subnet, name_subnet)
+			ip := docker_endpoint.IPAddress
+			mac := docker_endpoint.MacAddress
+			endpoint_id := docker_endpoint.EndpointID
+			interface_name_subnet := "cali" + endpoint_id[:mathutils.MinInt(11, len(endpoint_id))]
+
+			Expect(ip).Should(HavePrefix("192.169."))
+
+			// Check that the endpoint is created in etcd
+			etcd_endpoint := GetEtcdString(fmt.Sprintf("/calico/v1/host/test/workload/libnetwork/libnetwork/endpoint/%s", endpoint_id))
+			Expect(etcd_endpoint).Should(MatchJSON(fmt.Sprintf(
+				`{"state":"active","name":"%s","active_instance_id":"","mac":"%s","profile_ids":["%s"],"ipv4_nets":["%s/32"],"ipv6_nets":[]}`,
+				interface_name_subnet, mac, name_subnet, ip)))
+
+			// Check the interface exists on the Host - it has an autoassigned
+			// mac and ip, so don't check anything!
+			DockerString(fmt.Sprintf("ip addr show %s", interface_name_subnet))
+
+			// Make sure the interface in the container exists and has the  assigned ip and mac
+			container_interface_string := DockerString(fmt.Sprintf("docker exec -i %s ip addr", name_subnet))
+			Expect(container_interface_string).Should(ContainSubstring(ip))
+			Expect(container_interface_string).Should(ContainSubstring(mac))
+
+			// Make sure the container has the routes we expect
+			routes := DockerString(fmt.Sprintf("docker exec -i %s ip route", name_subnet))
+			Expect(routes).Should(Equal("default via 169.254.1.1 dev cali0 \n169.254.1.1 dev cali0 scope link"))
 
 			// Delete container and network
 			DockerString(fmt.Sprintf("docker rm -f %s", name_subnet))
@@ -264,7 +310,7 @@ var _ = Describe("Libnetwork Tests", func() {
 
 		It("creates a container with labels, but do not expect those in endpoint", func() {
 			// Create a container that will just sit in the background
-			DockerString(fmt.Sprintf("docker run --net %s -tid --label org.projectcalico.label.foo=bar --label org.projectcalico.label.baz=quux --name %s busybox", name, name))
+			DockerString(fmt.Sprintf("docker run --net %s -tid --label org.projectcalico.label.foo=bar --label org.projectcalico.label.baz=quux --name %s %s", name, name, os.Getenv("BUSYBOX_IMAGE") ))
 
 			// Gather information for assertions
 			docker_endpoint := GetDockerEndpoint(name, name)
@@ -276,7 +322,7 @@ var _ = Describe("Libnetwork Tests", func() {
 			// Check that the endpoint is created in etcd
 			etcd_endpoint := GetEtcdString(fmt.Sprintf("/calico/v1/host/test/workload/libnetwork/libnetwork/endpoint/%s", endpoint_id))
 			Expect(etcd_endpoint).Should(MatchJSON(fmt.Sprintf(
-				`{"state":"active","name":"%s","mac":"%s","profile_ids":["%s"],"ipv4_nets":["%s/32"],"ipv6_nets":[]}`,
+				`{"state":"active","name":"%s","active_instance_id":"","mac":"%s","profile_ids":["%s"],"ipv4_nets":["%s/32"],"ipv6_nets":[]}`,
 				interface_name, mac, name, ip)))
 
 			// Delete container
@@ -285,6 +331,54 @@ var _ = Describe("Libnetwork Tests", func() {
 
 	})
 
-	//docker stop/rm - stop and rm are the same as far as the plugin is concerned
+	Describe("docker run ipv6", func() {
+		var name string
+		BeforeEach(func() {
+			name = fmt.Sprintf("run%d", rand.Uint32())
+			DockerString(fmt.Sprintf("docker network create --ipv6 %s -d calico --ipam-driver calico-ipam", name))
+		})
+		AfterEach(func() {
+			DockerString(fmt.Sprintf("docker network rm %s", name))
+		})
+
+		It("creates a container on a network  and checks all assertions", func() {
+			// Create a container that will just sit in the background
+			DockerString(fmt.Sprintf("docker run --net %s -tid --name %s %s", name, name, os.Getenv("BUSYBOX_IMAGE") ))
+
+			// Gather information for assertions
+			docker_endpoint := GetDockerEndpoint(name, name)
+			ip := docker_endpoint.IPAddress
+			ipv6 := docker_endpoint.GlobalIPv6Address
+			mac := docker_endpoint.MacAddress
+			endpoint_id := docker_endpoint.EndpointID
+			interface_name := "cali" + endpoint_id[:mathutils.MinInt(11, len(endpoint_id))]
+
+			// Check that the endpoint is created in etcd
+			etcd_endpoint := GetEtcdString(fmt.Sprintf("/calico/v1/host/test/workload/libnetwork/libnetwork/endpoint/%s", endpoint_id))
+			Expect(etcd_endpoint).Should(MatchJSON(fmt.Sprintf(
+				`{"state":"active","name":"%s","active_instance_id":"","mac":"%s","profile_ids":["%s"],"ipv4_nets":["%s/32"],"ipv6_nets":["%s/128"]}`,
+				interface_name, mac, name, ip, ipv6)))
+
+			// Check the interface exists on the Host - it has an autoassigned
+			// mac and ip, so don't check anything!
+			DockerString(fmt.Sprintf("ip addr show %s", interface_name))
+			DockerString(fmt.Sprintf("ip -6 addr show %s", interface_name))
+
+			// Make sure the interface in the container exists and has the  assigned ipv6 and mac
+			container_interface_string := DockerString(fmt.Sprintf("docker exec -i %s ip addr", name))
+			Expect(container_interface_string).Should(ContainSubstring(ipv6))
+			Expect(container_interface_string).Should(ContainSubstring(mac))
+
+			// Make sure the container has the routes we expect
+			routes := DockerString(fmt.Sprintf("docker exec -i %s ip route", name))
+			Expect(routes).Should(Equal("default via 169.254.1.1 dev cali0 \n169.254.1.1 dev cali0 scope link"))
+			routes6 := DockerString(fmt.Sprintf("docker exec -i %s ip -6 route", name))
+			Expect(routes6).Should(MatchRegexp("default via fe80::.* dev cali0  metric 1024"))
+
+			// Delete container
+			DockerString(fmt.Sprintf("docker rm -f %s", name))
+		})
+	})
+	//Docker stop/rm - stop and rm are the same as far as the plugin is concerned
 	// TODO - check that the endpoint is removed from etcd and that the  veth is removed
 })
