@@ -3,7 +3,7 @@ package log
 import (
 	"encoding/json"
 
-	logger "github.com/Sirupsen/logrus"
+	logger "github.com/sirupsen/logrus"
 )
 
 func JSONMessage(formattedMessage string, data interface{}) {
